<?php

/* html_functions2.php: Some functions for html generation, only used by the
                        message.php page
*
    Copyright (C) 2002-2005  Hastymail Development group

    This file is part of Hastymail.

    Hastymail is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Hastymail is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Hastymail; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

* $Id: html_functions_2.php,v 1.50 2005/12/26 21:55:43 sailfrog Exp $
*/


/**
 *  function to print out the attachments part of the message page
 *
 *  @param  $structure      array       the message structure array
 *  @param  $part           float       message part number
 *  @param  $atag           string      the begining of a really long link tag
 *  @param  $level          int         keeps track of recursive calling for indentation
 *  @param  $types          array       viewable message types
 *  @param  $message_parts  array       message parts currently being viewed
 *  @param  $simple         boolean     simple mode flag
 *  @return                 string      html for the attachments section
 *
 */
function hm_print_attachments ($structure, $atag, $level=0, $types, $message_parts,
                               $output_rows=array()) {
   
    $str = $_SESSION['str'];
    foreach ($structure as $index => $array) {
        $output_rows[$index] = array();
        $double_link = false;
        if (!isset($array['type'])) {
            $type = 'unknown';
        }
        else {
            $type = $array['type'];
        }
        if (!isset($array['subtype'])) {
            $subtype = 'unknown';
        }
        else {
            $subtype = $array['subtype'];
        }
        $output_rows[$index]['type'] = $type;
        $output_rows[$index]['subtype'] = $subtype;
        $output_rows[$index]['double_link'] = false;
        $output_rows[$index]['atag'] = $atag;
        $output_rows[$index]['lbl_rfc822'] = $str[305];
        $output_rows[$index]['lbl_view'] = $str[39];
        $output_rows[$index]['lbl_download'] = $str[38];
        /* normal attachments */
        if ($type != 'mixed' && $type != 'unknown' && $subtype != 'rfc822' &&
            $subtype != 'digest') {
            $output_rows[$index]['rfc822'] = false;

            /* view links and checkboxes for supported types */
            $output_rows[$index]['checked'] = false;
            foreach ($message_parts as $part) {
                if (strcmp($index, $part) === 0) {
                    $output_rows[$index]['checked'] = true;
                    break;
                }
            }
            if (in_array($type.$subtype, $types) || $type == 'text') {
                $output_rows[$index]['viewable'] = true;
                $output_rows[$index]['from'] = false;
                $output_rows[$index]['double_link'] = true;
            }
            else {
                $output_rows[$index]['from'] = false;
                $output_rows[$index]['viewable'] = false;
            }
            /* download links for supported types */
            if ($type != 'message' || $type.$subtype == 'messagedelivery-status' ||
                $type.$subtype == 'messagerfc822-headers' || $type.$subtype ==
                'messageexternal-body' || $type.$subtype == 
                'messagedisposition-notification') {
                $output_rows[$index]['download'] = true;
                switch (true) {
                    case (isset($array['name']) && trim($array['name']) != ''):
                        $output_rows[$index]['name'] = urlencode($array['name']);
                        break;
                    case $type.$subtype == 'textplain':
                    case $type.$subtype == 'textenriched':
                        $output_rows[$index]['name'] = 'message.txt';
                        break;
                    case $type.$subtype == 'texthtml':
                        $output_rows[$index]['name'] = 'message.html';
                        break;
                    default:
                        $output_rows[$index]['name'] = 'unkown';
                        break;
                }
                if (isset($array['encoding']) && isset($array['encoding'])) {
                    $output_rows[$index]['encoding'] = strtolower($array['encoding']);
                }
                else {
                    $output_rows[$index]['encoding'] = false;
                }
            }
            /* indent and type section */
            $output_rows[$index]['indent_level'] = $level*8;
            switch (true) {
               case (isset($array['description']) && strtolower($array['description'])
                    != 'nil' && trim($array['description']) != ''):
                    $output_rows[$index]['description'] = hm_decode_field(str_replace('"',
                                                           '', $array['description']));
                    break;
                case (isset($array['subject']) && strtolower($array['subject']) != 'nil'
                     && trim($array['subject']) != ''):
                    $output_rows[$index]['description'] = hm_decode_field(str_replace('"',
                                                           '', $array['subject']));
                    break;
                case (isset($array['name']) && trim($array['name']) != ''):
                    $output_rows[$index]['description'] = hm_decode_field(str_replace('"',
                                                           '', $array['name']));
                    break;
                default:
                    $output_rows[$index]['description'] = false;
                    break;
            }
            /* size */
            if (isset($array['size'])) {
                $output_rows[$index]['size'] = $array['size'];
            }
            else {
                $output_rows[$index]['size'] = 'Unknown';
            }
        }
        /* rfc822 messages */
        if ($type == 'message' && ($subtype == 'rfc822' || $subtype == 'digest')) {
            $output_rows[$index]['rfc822'] = true;
            $output_rows[$index]['download'] = false;
            $output_rows[$index]['indent_level'] = $level*8;
            $output_rows[$index]['checked'] = false;
            $output_rows[$index]['viewable'] = false;
            if (isset($array['subject']) && 
                strtolower($array['subject']) != 'nil') {
                $output_rows[$index]['description'] = hm_htmlize(hm_decode_field(
                                                      stripslashes( $array['subject'])));
            }
            elseif (isset($array['description']) && 
                strtolower($array['description']) != 'nil') {
                $output_rows[$index]['description'] = hm_htmlize(hm_decode_field(
                                                   stripslashes( $array['description'])));
            }
            else {
                $output_rows[$index]['description'] = false;
            }
            if (isset($array['from']) && strtolower($array['from']) != 'nil') {
                $output_rows[$index]['raw_from'] = $array['from'];
                $output_rows[$index]['from'] = hm_htmlize(hm_decode_field(
                                               $array['from']));
            }
            $output_rows[$index]['size'] = $array['size'];
        }
        if (isset($array['subs'])) {
            if (isset($level)) {
                $level += 1;
            }
            $output_rows = hm_print_attachments ($array['subs'], $atag, $level, $types,
                                  $message_parts, $output_rows);
            $level -= 1;
        }
    }
    return $output_rows;
}

/**
 *  function to build the first 2/3 of the top and bottom of the message page
 *
 *  @param  $sort_info      array       the sort information for this mailbox
 *  @param  $mailbox        string      the currently selected folder
 *  @param  $imap_stream    resource    connection to imap
 *  @param  $page_num       int         he current page number if known
 *  @param  $id             string      session id
 *  @param  $uid            int         message uid
 *  @param  $dsp_folders    array       folder array formatted for display
 *  @param  $simple         boolean     flag indicating simple mode
 *  @return                 array       (html, prev/next toggle, current message flags)
 *
 */
function hm_message_header ($sort_info, $mailbox, $imap_stream, 
                            $page_num, $id, $uid, $dsp_folders, $simple=false) {
    $pre = false;
    $nex = false;
    $string = false;
    $pre_new = false;
    $nex_new = false;
    $key = array_keys($sort_info[2], $uid);
    $key = $key[0];
    $result = '';
    $str = $_SESSION['str'];
    $result = '';

    if (!in_array($uid, $sort_info[1][$page_num][2])) {
        if (isset($sort_info[1][$page_num + 1][2]) &&
            in_array($uid, $sort_info[1][$page_num + 1][2])) {
            $page_num = $page_num + 1;
        }
        elseif (isset($sort_info[1][$page_num - 1][2]) &&
                in_array($uid, $sort_info[1][$page_num - 1][2])) {
            $page_num = $page_num - 1;
        }
        else {
            foreach ($sort_info[1] as $index => $array) {
                if (in_array($uid, $array[2])) {
                    $page_num = $index;
                    break;
                }
            }
        }
    }
    $result .= '<input type="hidden" value="'.$page_num.'" name="page_num">'.
               hm_create_next_previous ($imap_stream, $sort_info[2], 
                                        urlencode($mailbox), $uid, 'normal', false,
                                        $str[9].' '.($page_num + 1), false, true, false);
    $mailbox_title = '&nbsp;&nbsp;&nbsp;<b>';
    if (isset($_SESSION['config']['utf7_decode'])) {
        $mailbox_title .= utf8_encode(imap_utf7_decode($dsp_folders[
                          urldecode($mailbox)]));
    }
    else {
        $mailbox_title .= $dsp_folders[urldecode($mailbox)];
    }
    $mailbox_title .= '</b>&nbsp;&nbsp;&nbsp;(&nbsp;'.$str[9].'&nbsp;'.($page_num + 1).
            ',&nbsp;&nbsp;'.$str[26].'&nbsp;'.($key+1).'&nbsp;/&nbsp;'.
            count($sort_info[2]).'&nbsp;)&nbsp;&nbsp;'."\n";
    return array($page_num, $result, $mailbox_title);
}

/**
 *  Create previous and next links for message ranges
 *
 *  @param      $imap_stream    resource    connection to imap
 *  @param      $range          array       uid array
 *  @param      $mailbox        string      current mailbox
 *  @param      $uid            int         current message id
 *  @param      $name           string      button name suffix
 *  @param      $multi_folder   boolean     true if the range spans multiple folders
 *  @param      $center         string      center button name
 *  @param      $thread         boolean     if this is prev/next in thread
 *  @param      $set_flag       boolean     cache read/unread flags for re-use
 *  @param      $use_flag       boolean     use cached flags
 *  @return                     string      html buttons
 *
 */
function hm_create_next_previous ($imap_stream, $range, $mailbox, $uid, $name,
                                   $multi_folder, $center, $thread, $set_flag, 
                                   $use_flag) {
    $pos = 0;
    $max = 0;
    if ($use_flag) {
        if (isset($_SESSION['prev_flag'])) {
            $prev_uid = $_SESSION['prev_flag'];
            $pos = 1;
            $prev_box = $mailbox;
        }
        if (isset($_SESSION['next_flag'])) {
            $next_uid = $_SESSION['next_flag'];
            $max = 2;
            $next_box = $mailbox;
        }
    }
    elseif (!$multi_folder) {
        if ($thread) {
            foreach($range[$mailbox] as $index => $array) {
                if (in_array($uid, $array[1])) {
                    $current_range = $array[1];
                    break;
                }
            }
        }
        else {
            $current_range = $range;
        }
        $current_index_array = array_keys($current_range, $uid);
        $pos = $current_index_array[0];
        $max = (count($current_range) - 1);
        if ($pos > 0) {
            $prev_uid = $current_range[($pos - 1)];
            $prev_box = $mailbox;
        }
        if ($pos < $max) {
            $next_uid = $current_range[($pos + 1)];
            $next_box = $mailbox;
        }
    }
    else {
        $range_index_array = array_keys($range);
        list($fold_pos) = array_keys($range_index_array, $mailbox);
        $fold_max = count($range_index_array) - 1;
        $current_index_array = array_keys($range[$mailbox]);
        $max = count($current_index_array) - 1;
        list($pos) = array_keys($current_index_array, $uid);
        if ($pos > 0) {
            $prev_uid = $current_index_array[($pos - 1)];
            $prev_box = $mailbox;
        }
        elseif ($fold_pos > 0) {
            $prev_box = $range_index_array[($fold_pos - 1)];
            $prev_index_array = array_keys($range[$prev_box]);
            $prev_uid = $prev_index_array[(count($prev_index_array) - 1)];
        }
        if ($pos < $max) {
            $next_uid = $current_index_array[($pos + 1)];
            $next_box = $mailbox;
        }
        elseif ($fold_pos < $fold_max) {
            $next_box = $range_index_array[($fold_pos + 1)];
            $next_index_array = array_keys($range[$next_box]);
            $next_uid = $next_index_array[0];
        }
    }
    $return_string = '';
    if ($pos > 0 || (isset($fold_pos) && $fold_pos > 0)) {
        if ($use_flag) {
            if (isset($_SESSION['prev_new']) && $_SESSION['prev_new'] == 1) {
                $new = 1;
                $style = 'style="{font-weight: 900}" ';
                $label = '&lt;&lt;&lt;';
            }
            else {
                $new = 0;
                $style = 'style="{font-weight: 100}" ';
                $label = '&lt;&lt;&lt;';
            }
        }
        else {
            if ($prev_box != $mailbox) {
                hm_select_mailbox($imap_stream, $prev_box);
            }
            $prev_flags = hm_get_flags($imap_stream, $prev_uid);
            if ($prev_box != $mailbox) {
                hm_select_mailbox($imap_stream, $mailbox);
            }
            if (stristr($prev_flags, '\Seen')) {
                $new = 0;
                $style = 'style="{font-weight: 100}" ';
                $label = '&lt;&lt;&lt;';
            }
            else {
                $style = 'style="{font-weight: 900}" ';
                $new = 1;
                $label = '&lt;&lt;&lt;';
            }
        }
        if ($set_flag) {
            $_SESSION['prev_flag'] = $prev_uid;
            $_SESSION['prev_new'] = $new;
        }
        $return_string .= '<input type="submit" '.$style.'name="previous_'. $name.
      '" value="'.$label.'" class="button3">'."\n";
        if ($name != 'delete' && $name != 'move') {
            $return_string .= '<input type="hidden" name="previous_uid_'.
                              $name.'" value="'.$prev_uid.
                              '"><input type="hidden" name="previous_box_'. $name.
                              '" value="'.$prev_box."\">\n";
        }
    }
    if ($center) {
        $_SESSION['current_flags'] = hm_get_flags($imap_stream, $uid);
        $return_string .= '<input type="submit" name="center_'.
                          $name.'" value="'.$center.'" class="button3">'."\n";
    }
    else {
        $return_string .= "&nbsp;";
    }
    if ($pos < $max || (isset($fold_pos) && isset($fold_max) && $fold_pos < $fold_max)) {
        if ($use_flag) {
            if (isset($_SESSION['next_new']) && $_SESSION['next_new'] == 1) {
                $new = 1;
                $style = 'style="{font-weight: 900}" ';
                $label = '&gt;&gt;&gt;';
            }
            else {
                $new = 0;
                $style = 'style="{font-weight: 100}" ';
                $label = '&gt;&gt;&gt;';
            }
        }
        else {
            if ($next_box != $mailbox) {
                hm_select_mailbox($imap_stream, $next_box);
            }
            $next_flags = hm_get_flags($imap_stream, $next_uid);
            if ($next_box != $mailbox) {
                hm_select_mailbox($imap_stream, $mailbox);
            }
            if (stristr($next_flags, '\Seen')) {
                $new = 0;
                $style = 'style="{font-weight: 100}" ';
                $label = '&gt;&gt;&gt;';
            }
            else {
                $new = 1;
                $style = 'style="{font-weight: 900}" ';
                $label = '&gt;&gt;&gt;';
            }
        }
        if ($set_flag) {
            $_SESSION['next_flag'] = $next_uid;
            $_SESSION['next_new'] = $new;
        }
        $return_string .= '<input type="submit" name="next_'.$name.
                          '" '.$style.'value="'.$label.'" class="button3">'."\n";
        if ($name != 'delete' && $name != 'move') {
            $return_string .= '<input type="hidden" name="next_uid_'.$name.
                              '" value="'.$next_uid.'"><input type="hidden" '.
                              'name="next_box_'.$name.'" value="'.$next_box.'">'."\n";
        }
    }
    return $return_string;    
}

/**
 *  Folder preview on the message page
 *
 *  @param      $imap_stream    resource    connection to imap
 *  @param      $cur_uid        int         message uid for the center of the list
 *  @param      $sort_info      array       sort information for thread view
 *  @param      $id             string      session id
 *  @param      $mailbox        string      current mailbox
 *  @param      $real_uid       int         the currently displayed message
 *  @return                     array       data for creating the folder preview
 */
function hm_folder_preview ($imap_stream, $cur_uid, $sort_info, $id, $mailbox,
                            $real_uid) {
    $output_array = array();
    $str            = $_SESSION['str'];
    $amount         = $_SESSION['settings']['folder_preview'];
    $check          = ($amount - 1)/2;
    $scroll_down    = false;
    $scroll_up      = false;
    if (in_array($cur_uid, $sort_info[2])) {
        list($index) = array_keys($sort_info[2], $cur_uid);
        if ($index !== false) {
            $total = count($sort_info[2]);
            if ($total <= $amount) {
                $start_pos = 0;
            }
            elseif ($index < $check) {
                $start_pos = 0;
            }
            else {
                $start_pos = $index - $check;
            }
            if (($start_pos + $amount) >= $total) {
                $start_pos = $total - $amount;
            }
            if ($start_pos < 0) {
                $start_pos = 0;
            }
            if ($index > $check) {
                $scroll_up = $start_pos;
            }
            if (($start_pos + $amount - 1) < ($total - 1)) {
                $scroll_down = $start_pos + $amount - 1;
            }
            $uid_slice   = array_slice($sort_info[2], $start_pos, $amount);
            $uid_string  = implode(',', $uid_slice);
            $uid_headers = &hm_get_messages ($uid_string, $imap_stream);
            if ($scroll_up) {
                $output_array['scroll_up'] = 'message.php?id='.urlencode($id).
                                                 '&amp;mailbox='.$mailbox.'&amp;'.
                                                 'scroll_uid='.$sort_info[2][$scroll_up].
                                                 '&amp;uid='.$real_uid;
            }
            else {
                $output_array['scroll_up'] = false;
            }
            $i = 0;
            foreach ($uid_slice as $uid) {
                $uid_current = false;
                if ($real_uid == $uid) {
                    $uid_current = true;
                }
                $r = hm_print_row2($uid_headers[$uid], false, $mailbox, 
                             $id, $uid, $sort_info, true, false, false, 
                             false, $uid_current, array(), false, false);
                $output_array['rows'][$i] = $r;
                $i++;
            }
            if ($scroll_down) {
                $output_array['scroll_down'] = 'message.php?id='.urlencode($id).
                                                 '&amp;mailbox='.$mailbox.'&amp;'.
                                                 'scroll_uid='.
                                                 $sort_info[2][$scroll_down].
                                                 '&amp;uid='.$real_uid;
            }
            else {
                $output_array['scroll_down'] = false;
            }
            $output_array['label'] = 
                '<b>&nbsp;&nbsp; '.$str[19].' '.$str[18].' '.($start_pos + 1).'-'.
                ($start_pos + $amount).' ('.$sort_info[0].')</b>
                &nbsp;&nbsp;&nbsp;<a class="font2" href="message.php?mailbox='.$mailbox.
                '&amp;id='.urlencode($id).'&amp;uid='.$real_uid.'&amp;hide_folder_preview=1">
                ('.$str[130].')</a>';
        }
    }
    return $output_array;
}

/**
 *  Display an image part
 *
 *  @param      $id             string          session id
 *  @param      $mailbox        string          message mailbox
 *  @param      $uid            int             message uid
 *  @param      $part           string          message part
 *  @param      $decode         string          decode method
 *  @param      $type           string          mime type
 *  @param      $subtype        string          mime subtype
 *  @param      $name           string          image filename
 *  @return                     string          html for an image part
 */
function hm_print_image_part ($id, $mailbox, $uid, $part, $decode, $type, $subtype,
                              $name) {
    return '      <br><center><img src="download.php?id='.$id.
         '&amp;mailbox='.$mailbox.
         '&amp;uid='.$uid.
         '&amp;part='.$part.
         '&amp;decode='.$decode.
         '&amp;type='.$type.
         '&amp;subtype='.$subtype.
         '&amp;name='.$name.'" alt="'.$name.'"></center><br>'."\n";
}

/**
 *  Display an HTML part
 *
 *  @paryyam    $text_part                  string          message part body
 *  @param      $message_charset_convert    boolean         charset conversion flag
 *  @param      $message_code_page          array           character conversion array
 *  @param      $message_charset            string          message charset
 *  @param      $message_headers            array           message headers
 *  @param      $id                         string          session id
 *  @return                                 string          output for an html part
 */
function hm_print_html_part ($text_part, $code_page, $charset, $message_headers, $id,
                             $structure, $mailbox, $uid) {  
    require_once('../filter/htmlfilter.inc');
    require_once('../filter/filter.php');
    $out = '';
    $str = $_SESSION['str'];
    $level = 1;
    if (isset($_SESSION['settings']['filter_level']) &&
        is_numeric($_SESSION['settings']['filter_level'])) {
        $level = $_SESSION['settings']['filter_level'];
    }
    $clean = hm_filter_html($text_part, $level);
    $clean = hm_special_chars2($clean, $charset, $code_page, true);

    /* if no html tags are found in the message aside from the 
       start and stop sanitized html tags inserted by the filter
       then format this as a text part and display as text */

    if (substr_count($clean, "<") < 3) {
        $clean = &hm_text_cleanup($clean);
        $clean = "\n<pre>\n".$clean."\n</pre>\n";
    } 
    /* the html filter puts new window targets in a tags, lets take them
       out if the href is a reference to the current page */
    $clean = preg_replace("/(\<a href=\"#.*) target=\"_blank\"/", "\\1", $clean);

    /* replace mailto: with a hm url */
    $clean = preg_replace("/\<a href=\"mailto:(.*)\" target=\"_blank\"\>/", 
                 "<a href=\"compose.php?id=".$id."&amp;to=\\1\">", $clean); 
    /* inline images */
    if (isset($_SESSION['settings']['filter_level']) &&
        ($_SESSION['settings']['filter_level'] == 2 || 
         $_SESSION['settings']['filter_level'] == 4)) {
            if (preg_match_all("/src=\"cid:([^\"]*)\"/", $clean, $matches)) {
                if (isset($matches[1]) && is_array($matches[1])) {
                    foreach ($matches[1] as $val) {
                        $key = hm_find_cid_part($structure, '<'.$val.'>');
                        if ($key) {
                            $atts = hm_in_multi_array($key, $structure);
                            $src = 'download.php?id='.$id.'&amp;mailbox='.$mailbox.
                                   '&amp;uid='.$uid.'&amp;part='.$key.'&amp;decode='.
                                   $atts[1]['encoding'].'&amp;type='.$atts[1]['type'].
                                   '&amp;subtype='.$atts[1]['subtype'].'&amp;name='.
                                   $atts[1]['name'];
                            $clean = str_replace('cid:'.$val, $src, $clean);
                        }
                }
            }
        } 
    }
    $out .= '      <table width="97%" align="center">
         <tr>
             <td>
             '.$clean.'<br>
             </td>
         </tr>
      </table>'."\n";
      return $out;
      
}

/**
 *  Display an HTML part
 *
 *  @param      $text_part                  string          message part body
 *  @param      $message_charset_convert    boolean         charset conversion flag
 *  @param      $message_code_page          array           character conversion array
 *  @param      $message_utf                boolean         utf8 conversion flag
 *  @param      $id                         string          session id
 *  @param      $uid                        int             message uid
 *  @param      $part                       string          part number of the message
 *  @param      $mailbox                    string          current mailbox
 *  @return                                 string          output for a text part
 */
function hm_print_text_part ($text_part, $code_page, $charset, $id, $uid, $part,
                             $mailbox) {
    $out = '';
    $str = $_SESSION['str'];
    if (isset($_SESSION['settings']['font_type'])) {
        $font_type = $_SESSION['settings']['font_type'];
    }
    else {
        $font_type = 'monospace';
    }
    if (isset($_SESSION['settings']['font_size'])) {
        if ($_SESSION['settings']['font_size'] < 3) {
            switch ($_SESSION['settings']['font_size']) {
                case 0:
                    $pt = 11;
                case 1:
                    $pt = 9;
                case 2:
                    $pt = 12;
                default:
                    $pt = 11;
            }
        }
        else {
            $pt = $_SESSION['settings']['font_size'];
        }
    }    
    else {
        $pt = 11;
    }
    if (stristr($charset, 'utf-8')) {
        $encode = false;
    }
    else {
        $encode = true;
    }
    $text_part = hm_special_chars2($text_part, $charset, $code_page, false, $encode);
    $lines = split("\n", $text_part);
    $text_part = '';
    $last_line = '';

    /* add reply quoting color */
    $block = false;
    foreach ($lines as $line) {
        switch (true) {
            case substr($line, 0, 4) == '&gt;':
            case $last_line          == "\\":
            case $line && $line{0}            == '|':
            case $line && $line{0}            == ']':
                $last_line = substr($line, -1, 1);
                if (!$block) { 
                    $line = "<font\tstyle=\"{font-family:\t".$font_type.
                            ";\tcolor:\t#555555;\tsize:\t".$pt."pt}\">".$line."<br>\n";
                    $block = true; 
                }
                else {
                    $line = $line."<br>\n";
                }
                break;
            default:
                if ($block) {
                    $line = '</font>'.$line."<br>\n";
                }
                else {
                    $line = $line."<br>\n";
                }
                $last_line = '';
                $block = false;
                break;
        }
        $line = str_replace(array('&gt;', '&lt;'), array(' &gt;', ' &lt;'), $line);
        $text_part .= $line;
    }

    /* add clickable (http|ftp|rtsp)s links */
    if (isset($_SESSION['settings']['click_links']) &&
        $_SESSION['settings']['click_links'] == 'true') {
        $link_regex = "/((http|ftp|rtsp)s?:\/\/(%[[:digit:]A-Fa-f][[:digit:]A-Fa-f]".
                      "|[-_\.!~\*';\/\?#:@&=\+$,[:alnum:]])+)/i";
        $link_replace = "<a\ttarget=\"_blank\"\ttitle=\"$str[333]\"\thref=\"\\1\">\\1</a>";
        $text_part = preg_replace($link_regex, $link_replace, $text_part);
    }
    $text_part = str_replace(array(' &lt;', ' &gt;'), array('&lt;', '&gt;'), $text_part);

    /* add clickable email addresses */
    if (isset($_SESSION['settings']['click_email']) &&
        $_SESSION['settings']['click_email'] == 'true') {
        $email_regex = "/([\w\.\-]+@[a-zA-Z0-9\-]+(\.[a-zA-Z0-9\-]{1,})*(\.".
                      "[a-zA-Z]{2,5}){1,2})/";
        $email_replace = "<a\ttitle=\"$str[332]\"\thref=\"compose.php?id=".$id.
                         "&amp;to=\\1\">\\1</a>";
        $text_part = preg_replace($email_regex, $email_replace, $text_part);
    }
    $text_part = str_replace(' ', '&nbsp;', $text_part);

    $out .= '      <table width="100%">
         <tr>
            <td nowrap width="1%">
            &nbsp;
            </td>
            <td style="{font-family: '.$font_type.'; font-size: '.$pt.'pt}">';

    /* check for LARGE text parts and display notice */
    if (isset($_SESSION['cutoff']) && $_SESSION['cutoff'] == 1) {
        $out .= '            <br>
            <table align="center" cellspacing="4" class="tables4">
               <tr>
                  <td align="center">
                  '.$str[260].'
                  </td>
               </tr>
               <tr>
                  <td align="center">
                  <a class="font2" href="message.php?id='.$id.'&amp;uid='.$uid.
                  '&amp;part='.$part.'&amp;mailbox='.$mailbox.'&amp;big_text=1">'.
                  $str[261].'</a><br>
                  </td>
               </tr>
            </table><br>'."\n";
    }

    /* display text part */
    $out .= "\n".'<!-- Message text -->'."\n".str_replace("\t", ' ', $text_part).
            '<!-- End of message text -->
            </td>
         </tr>
      </table>'."\n";
    return $out;
}
?>
