<?php

/* session.php: Functions to maintain the session.
*
    Copyright (C) 2002-2005  Hastymail Development group

    This file is part of Hastymail.

    Hastymail is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Hastymail is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Hastymail; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

* $Id: session.php,v 1.289 2006/01/06 21:06:43 sailfrog Exp $
*/

/**
 *  validates vars by type
 *
 *  @param      $_POST      array       the post array values
 *  @param      $_GET       array       the get array values
 *  @param      $page       string      the current page
 *  @return                 array       validated array values
 *
 */
function hm_validate_input($_POST, $_GET, $page) {
    $v_types = require_once('post_and_get.php');
    $checked = array();
    $input = array();
    if (!empty($_POST)) {
        $input = $_POST;
    }
    elseif (!empty($_GET)) {
        $input = $_GET;
    }
    if (!empty($input)) {
        foreach ($input as $key => $value) {
            if (isset($v_types[$page][$key])) {
                switch ($v_types[$page][$key]) {
                    case 'true':
                        $checked[$key] = 1;
                        break;
                    case 'string':
                        if (ini_get('magic_quotes_gpc') == 1) {
                            $checked[$key] = stripslashes((string) $value);
                        }
                        else {
                            $checked[$key] = (string) $value;
                        }
                        break;
                    case 'id':
                        if (strlen($value) == 32 && preg_match("/[a-zA-Z0-9]/", $value)) {
                            $checked[$key] = $value;
                        }
                        else {
                            $checked[$key] = false;
                        }
                        break;
                    case 'folder':
                            $checked[$key] = (string) $value;
                        break;
                    case 'int':
                            $checked[$key] = (int) intval($value);
                        break;
                    case 'float':
                        $checked[$key] =  preg_replace( "/[^0-9\.]*/", '', $value );
                        break;
                    case 'array':
                        if (is_array($value)) {
                            $checked[$key] = $value;
                        }
                        break;
                    default:
                        break;
                }
            }
        }
    }
    if (!isset($checked['mailbox'])) {
        $checked['mailbox'] = 'INBOX';
    }
    if (!isset($checked['id'])) {
        $checked['id'] = false;
    }
    return $checked;
}

/**
 *  init function for hastymail login
 *
 *  @param  $hm_input       array       the POST vars passed to this page
 *  @param  $config_file    string      the path+name of the site config file
 *  @param  $hasty_key      string      encryption key
 *  @return                 resource    a connection to the IMAP server
 *
 */
function hm_hastymail_login ($hm_input, $config_file, $hasty_key) {
    $alt_index = false;
    if (isset($hm_input['alt_server']) && $hm_input['alt_server'] > 0) {
        $alt_index = $hm_input['alt_server'];
    }
    set_magic_quotes_runtime(0);
    /* open config file */
    $config = hm_get_config($config_file, $alt_index);
    if ($hm_input['pass'] != '' && $hm_input['user'] != '') {
        $pass = hmw_crypt($hm_input['pass'], $config, 'encrypt', $hasty_key);
        list ($hm_input['user'], $config['domain']) = hm_get_domain_and_username($config,
                                                    $hm_input['user']);
        $debug = false;
        if (isset($config['login_debug']) && $config['login_debug'] == 'true') {
            $debug = true;
        }
        if ($config['imap_server'] == 'dynamic' && 
            isset($config['dynamic_imap_domain']) &&
            $config['dynamic_imap_domain'] != '' &&
            $config['dynamic_imap_domain'] != 'false') {
            $config['imap_server'] = $hm_input['user'].'.'.
                                     $config['dynamic_imap_domain'];
        }
        $config = hm_custom_imap_server($hm_input['user'], $config);
        /* check username/password with imap server */
        if ($imap_stream = hm_imap_login($hm_input['user'], $pass, $config, $hasty_key, 
                                         false, $debug)) {
            ini_set('output_buffering', '0');
            ini_set('url_rewriter.tags', '');
            ob_start();
            if (ini_get('session.use_cookies') == 1) {
                ini_set('session.use_cookies', '0');
            }
            /* setup session vars */
            $session_id = md5(uniqid(rand(),1));
            session_id($session_id);
            session_start();
            $_SESSION['notices'] = array();
            $hm_input['id'] = $session_id;
            $_SESSION['counter'] = 1;
            $_SESSION['config'] = $config;
            $_SESSION['user'] = $hm_input['user'];
            $_SESSION['pass'] = $pass;
            $_SESSION['ip_addy'] = $_SERVER['REMOTE_ADDR'];
            $_SESSION['user_agent'] = $_SERVER['HTTP_USER_AGENT'];
            $_SESSION['user_id'] = $_SERVER['REMOTE_ADDR'].$_SERVER['HTTP_USER_AGENT'];
            $_SESSION['settings'] = hmw_get_settings($hm_input['user'], $config);
            if (isset($config['delim']) && $config['delim'] != '') {
                $_SESSION['delimiter'] = $config['delim'];
            }
            if (isset($_SESSION['settings']['folded'])) {
                $_SESSION['default_folded'] = $_SESSION['settings']['folded'];
            }
            else {
                $_SESSION['default_folded'] = array();
            }
            if (isset($config['cookie_check']) && $config['cookie_check'] == 'true') {
                $rand_id = md5(uniqid(rand(),1));
                setcookie('hm_id', $rand_id);
                $_SESSION['cookie_id'] = $rand_id;
            }
            /* get i18n strings */
            if (isset($config['default_lang'])) {
                if (isset($_SESSION['settings']['lang'])) {
                    $str = hm_translate($_SESSION['settings']['lang']);
                }
                else {
                    $str = hm_translate($config['default_lang']);
                }
            }
            elseif (isset($_SESSION['settings']['lang'])) {
                $str = hm_translate($_SESSION['settings']['lang']);
            }
            else {
                $str = hm_translate('en_US');
            }
            if (isset($config['utf7_decode_folder']) &&
                $config['utf7_decode_folder'] == 'true' &&
                function_exists('imap_utf7_decode')) {
                $_SESSION['config']['utf7_decode'] = 'true';
            }
            if (function_exists('mb_convert_encoding')) {
                $_SESSION['mb'] = true;
            }
            else {
                $_SESSION['mb'] = false;
            }
            $_SESSION['charsets'] = include('../lang/charsets.php');
            foreach ($str as $key => $value) {
                if (is_array($value)) {
                    foreach ($value as $k => $v) {
                        $_SESSION['str'][$key][$k] = hm_special_chars2($v,
                        $str['charset'], $_SESSION['charsets'][$str['charset']]);
                    }
                }
                else {
                    $_SESSION['str'][$key] = hm_special_chars2($value, $str['charset'], 
                    $_SESSION['charsets'][$str['charset']]);
                }
            }
            $_SESSION['str']['charset'] = 'UTF-8';
            $_SESSION['caps'] = hm_get_caps($imap_stream);
            if (isset($config['log_users']) && $config['log_users'] == 'true') {
                hm_log_users($config['log_path'], $hm_input['user'], 'login');
            }
        }

        /* bail on bad login */
        else {
            if (isset($config['log_users']) && $config['log_users'] == 'true') {
                hm_log_users($config['log_path'], $hm_input['user'], 'failed');
            }
            header("Location: ".$config['hostname'].$config['install_path'].
                   "logout.php");
            exit;
        }
    }
    else {
        header("Location: ".$config['hostname'].$config['install_path'].  "logout.php");
        exit;
    }
    return $imap_stream;
}

/**
 *  reads in the site config file 
 *
 *  @param  $config_file    string      the config file name with absolute path
 *  @param  $alt            boolean     alternate imap server flag
 *  @return                 array       configuration file values
 *
*/
function hm_get_config($config_file, $alt=false, $page_prefix=false) {
    if (!is_readable($config_file)) {
        hm_output_page(array('config_found' => false), 'login_template.php', 'default',
                       $page_prefix);
         exit;
    }
    $config = array();
    $config_fh = fopen ($config_file, 'r');
    while (!feof ($config_fh)) {
        $line = fgets($config_fh, 1024);
        if ($line{0} != '#') {
            $value = substr(trim(strstr($line, '=')), 1);
            $offset = (strlen(trim($line)) - strlen($value)) - 1;
            $name = substr($line, 0, $offset);
            if ($name) {
                $config[$name] = $value; 
            }
        }
    }
    fclose($config_fh);
    if ($alt) {
        foreach($config as $name => $value) {
            if (substr($name, 0, 6) == 'alt_'.$alt.'_') {
                $config[substr($name, 6)] = $value;
            }
        }
    }
    if (isset($_SERVER['SERVER_PORT'])) {
        if ($_SERVER['SERVER_PORT'] == '443') {
            $config['hostname'] = 'https://';
        }
        elseif ($_SERVER['SERVER_PORT'] == '80') {
            $config['hostname'] = 'http://';
        }
        else {
            if (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] == 'on') {
                $config['hostname'] = 'https://';
            }
            else {
                $config['hostname'] = 'http://';
            }
        }
    }
    $config['install_path'] = rtrim(dirname($_SERVER['PHP_SELF']), '/').'/';
    if (isset($config['webserver_name']) && $config['webserver_name'] != '') {
        $config['hostname'] .= $config['webserver_name'];
    }
    else {
        if (isset($config['use_server_name']) && $config['use_server_name'] == 'true' &&
            isset($_SERVER['SERVER_NAME'])) {
            $config['hostname'] .= $_SERVER['SERVER_NAME'];
        }
        else {
            $config['hostname'] .= $_SERVER['HTTP_HOST'];
        }
    }
    return $config;
}

/**
 *  clean up session on logout
 *
 *  @param      $id             string      session id
 *  @param      $config_file    string      absolute path to the hastymail config file
 *  @return                     void
 *
 */
function hm_session_logout ($id, $config_file) {
    if (!$id) {
        hm_logged_out($config_file);
    }
    ini_set('output_buffering', '0');
    if (ini_get('session.use_cookies') == 1) {
        ini_set('session.use_cookies', '0');
    }
    set_magic_quotes_runtime(0);
    ini_set('url_rewriter.tags', '');
    ob_start();
    session_id($id);
    session_start($id);
    $config = hm_get_config($config_file);
    if (isset($config['log_users']) && $config['log_users'] == 'true' &&
        isset($_SESSION['user'])) {
        hm_log_users($config['log_path'], $_SESSION['user'], 'logout');
    }
    if (isset($_SESSION['attachments'])) {
        foreach ($_SESSION['attachments'] as $name => $vals) {
            if (file_exists($_SESSION['config']['settings_dir']."attachments/".$name)) {
                unlink($_SESSION['config']['settings_dir']."attachments/".$name);
                clearstatcache();
            }
        }
    }
    $_SESSION = array();
    session_destroy();
}

/**
 *  continue the session
 *
 *  @param  $id             string      session id
 *  @param  $config_file    string      absolute path to the hastymail config file
 *  @param  $clear          boolean     remove attachments from session if they exist
 *  @param  $skip           boolean     flag to skip dynamic session_id
 *  @param  $new_id         boolean     if true generate a new session_id
 *  @return                 boolean     redirects to logout on failure
 *
*/
function hm_session_continue ($id, $config_file, $clear=true, $skip=false,
                              $new_id=false) {
    if (!$id) {
        hm_logged_out($config_file);
    }
    ini_set('output_buffering', '0');
    if (ini_get('session.use_cookies') == 1) {
        ini_set('session.use_cookies', '0');
    }
    set_magic_quotes_runtime(0);
    ini_set('url_rewriter.tags', '');
    session_id($id);
    session_start($id);
    if (!isset($_SESSION['user_id']) ||
        $_SESSION['user_id'] != $_SERVER['REMOTE_ADDR'].$_SERVER['HTTP_USER_AGENT']) {
        hm_logged_out($config_file);
    }
    if ($clear && isset($_SESSION['attachments'])) {
        foreach ($_SESSION['attachments'] as $name => $vals) {
            if (file_exists($_SESSION['config']['settings_dir']."attachments/".$name)) {
                unlink($_SESSION['config']['settings_dir']."attachments/".$name);
                clearstatcache();
            }
        }
        $_SESSION['attachments'] = array();
        unset($_SESSION['attachments']);
    }
    if (isset($_SESSION['config']['dynamic_sessionid']) &&
        $_SESSION['config']['dynamic_sessionid'] == 'true') {
        $new_id = true;
    }
    if (!$skip && $new_id) {
        $session_values = $_SESSION;
        $_SESSION = array();
        session_unset();
        session_destroy();
        $session_id = md5(uniqid(rand(),1));
        session_id($session_id);
        session_start($session_id);
        $_SESSION = $session_values;
    }
    if (isset($_SESSION['config']['cookie_check']) && 
        $_SESSION['config']['cookie_check'] == 'true' && !$skip) {
        $cookie_id = $_COOKIE['hm_id'];
        $session_check = $_SESSION['cookie_id'];
        if ($cookie_id != $session_check) {
            hm_logged_out($config_file);
        }
        $rand_id = md5(uniqid(rand(),1));
        setcookie('hm_id', $rand_id);
        $_SESSION['cookie_id'] = $rand_id;
    }
    return true;
}

/**
 *  encrypt/decrypt user password with PHP mycrypt lib
 *
 *  @param      $input      string      password to encrypt/decrypt
 *  @param      $type       string      encrypt or decrypt flag
 *  @param      $cipher     string      encryption cipher to use
 *  @param      $key        string      encryption key
 *  @return                 string      encrypted/decrypted string
 */    
function hm_mcrypt($input, $type, $cipher, $key) {
    if (isset($_SERVER['HASTY_KEY'])) {
        $key = $_SERVER['HASTY_KEY'];
    }
    else {
        if ($key == 'You should be using the more secure version!') {
            return false; 
        }
    }
    $result = false;
    $td = mcrypt_module_open ($cipher, '', 'ecb', '');
    $key = substr ($key, 0, mcrypt_enc_get_key_size ($td));
    $iv_size = mcrypt_enc_get_iv_size ($td);
    $iv = mcrypt_create_iv ($iv_size, MCRYPT_RAND);
    if (mcrypt_generic_init ($td, $key, $iv) != -1) {
        if ($type == 'encrypt') {
            $result = mcrypt_generic ($td, $input);
            mcrypt_generic_deinit ($td);
            mcrypt_module_close ($td);
        }
        elseif ($type == 'decrypt') {
            $result = mdecrypt_generic ($td, $input);
            mcrypt_generic_deinit ($td);
            mcrypt_module_close ($td);
        }
    }
    else {
        return false;
    }
    if ($type == 'decrypt') {
        $result = rtrim($result, "\0");
    }
    return $result;
}

/**
 *  RC4 based encrypt/decrypt function
 *
 *  @param      $input      string  the string to encrypt or decrypt
 *  @param      $key        encryption key
 *  @return                 string  encrypted/decrypted string
 *
*/
function hm_crypt($input, $key) {
    if (isset($_SERVER['HASTY_KEY'])) {
        $key = $_SERVER['HASTY_KEY'];
    }
    else {
        if ($key == 'You should be using the more secure version!') {
            return false; 
        }
    }
    $k_tmp = preg_split('//', $key, -1, PREG_SPLIT_NO_EMPTY);
    foreach($k_tmp as $char) {
        $k[] = ord($char);
    }
    unset($k_tmp); 
    $message = preg_split('//', $input, -1, PREG_SPLIT_NO_EMPTY);
    $rep = count($k);
    for ($n=0;$n<$rep;$n++) {
        $s[] = $n;
    }
    $i = 0;
    $f = 0;
    for ($i = 0;$i<$rep;$i++) {
        $f = (($f + $s[$i] + $k[$i]) % $rep);
        $tmp = $s[$i];
        $s[$i] = $s[$f];
        $s[$f] = $tmp;
    }
    $i = 0;
    $f = 0;
    foreach($message as $letter) {
        $i = (($i + 1) % $rep);
        $f = (($f + $s[$i]) % $rep);
        $tmp = $s[$i];
        $s[$i] = $s[$f];
        $s[$f] = $tmp;
        $t = $s[$i] + $s[$f];
        $done = ($t^(ord($letter)));
        $i++;
        $f++;
        $enc_array[] = chr($done);
    }
    $coded = implode('', $enc_array);
    return $coded;
}

/**
 *  log the user out
 *
 *  @param  $config_file    string      hastymail configuration file with absolute path
 *  @param  $id             string      session id
 *  @return                 void        prints logout message and exits the calling script
 *
 */
function hm_logged_out ($config_file, $id=false) {
    $config = hm_get_config($config_file);
    $page_data = array();
    if (isset($config['default_lang'])) {
        $str = hm_translate($config['default_lang']);
    }
    else {
        $str = hm_translate('en_US');
    }
    if (isset($config['default_theme']) && $config['default_theme'] != '') {
        $style = str_replace(array('_large', '_small'), '', $config['default_theme']);
    }
    else {
        $style = 'default.css';
    }
    $page_data['style'] = $style;
    $page_data['lbl_logout'] = $str[195];
    $page_data['lbl_logout_link'] = $str[196];
    $page_data['charset'] = $str['charset'];
    $page_data['title'] = $str[97];
    $page_data['hostname'] = $config['hostname'];
    $page_data['install_path'] = $config['install_path'];
    if (isset($_SERVER['HTTP_USER_AGENT']) && 
        hm_browser_type ($_SERVER['HTTP_USER_AGENT'], true) ||
        (isset($config['always_simplify']) && $config['always_simplify'] == 'true')) {
        hm_output_page($page_data, 'loggedout_template.php', 'simple');
    }
    else {
        hm_output_page($page_data, 'loggedout_template.php', 'default');
    }
    exit; 
}

/**
 *  logs user login/outs to specified file
 *
 *  @param      $path       string      the full path and filename to the log file
 *  @param      $username   string      the users login name
 *  @param      $type       string      login or logout
 *  @return                 void        fails silently if there is a problem
 *
 */
function hm_log_users($path, $username, $type) {
    clearstatcache();
    if (is_writable($path) && file_exists($path) && $log_fh = @fopen($path, 'a+')) {
        $string = "$type\t$username\t".$_SERVER['REMOTE_ADDR'].
            "\t".date("F j, Y, g:i a");
        if (isset($_SERVER['HTTP_USER_AGENT'])) {
            $string .= "\t".$_SERVER['HTTP_USER_AGENT'];
        }
        $string .= "\n";
        @fputs($log_fh, $string);
        @fclose($log_fh);
    }
    else {
        if ($type == 'login') {
            $_SESSION['notices'][] = $_SESSION['str'][343];
        }
    }
}

/**
 *  translation function to get the interface strings.
 *
 *  @param  $lang           string      the selected language
 *  @param  $lang_options   boolean     indicates language selection 
 *  @return                 array       translated strings array
 *
 */
function hm_translate($lang, $lang_options=false) {
    $strings = array();
    if (!$lang_options) {
        require_once('../lang/lang_index.php');
    }
    if (isset($lang_options[$lang]) && is_readable('../lang/'.$lang.'.php')) {
        $strings = require_once('../lang/'.$lang.'.php');
    }
    else {
        $strings = require_once('../lang/en_US.php');
    }
    return $strings; 
}

/**
 *  Custom function to get imap server ip based on username
 *
 *  @param      $user       string      username 
 *  @param      $config     array       site config vals
 *  @return                 array       updated config vals
 *
 */
function hm_custom_imap_server($user, $config) {
    /*
        The purpose of this function is to allow sites to easily create
        there own look up function to dynamicly determine which IMAP server
        to use based on the username. It is called once during the hastymail
        login function (located in this file (hm_hastymail_login). There are 3
        values that you may want to modify in this function:

        $config['imap_server']      <- hostname or ip address of IMAP server
        $config['imap_port']        <- port to connect on
        $config['imap_ssl']         <- allowed values are ssl, tls, or false

        Any value that you do not set will default to the setting from your
        hastymail.conf file. All you need to do is connect to your db or open
        a lookup file, get the imap server for the given username and set
        the config value like so:

        $config['imap_server'] = 'your value';

        Make sure you leave this last line, which returns the modified (or
        unmodified) config values.
    */
    return $config;
}

/**
 *  Output a page using the selected template and template set
 *
 *  @param      $page_data      array       all the data for the template
 *  @param      $template       string      template for this page
 *  @param      $template_set   string      template set to use
 *  @param      $page_prefix    boolean     work around for top level dir
 *  @return                     void        returns nothing, output pages
 *
 */
function hm_output_page($page_data, $template, $template_set, $page_prefix=false) {

    /* make sure selected template set exists */
    if (!is_dir('../templates/'.$template_set)) {
        $template_set = 'default';
    }

    /* make all vals local so names are shorter in the template */
    foreach ($page_data as $name => $value) {
        $$name = $value;
    }

    /* Open the template and output the page */
    if ($page_prefix) {
        require_once($page_prefix.'templates/'.$template_set.'/'.$template);
    }
    else {
        require_once('../templates/'.$template_set.'/'.$template);
    }
}

?>
